@@ -80,7 +80,7 @@ class EventManager:
 
         debug.println(debug.LEVEL_INFO, 'EVENT MANAGER: Dectivating', True)
         self._active = False
-        for eventType in list(self._scriptListenerCounts.keys()):
+        for eventType in self._scriptListenerCounts.keys():
             self.registry.deregisterEventListener(self._enqueue, eventType)
         self._scriptListenerCounts = {}
         self.deregisterKeystrokeListener(self._processKeyboardEvent)
@@ -165,6 +165,14 @@ class EventManager:
             debug.println(debug.LEVEL_INFO, msg, True)
             return True
 
+        if event.type.startswith('object:state-changed:sensitive'):
+            if role in [pyatspi.ROLE_MENU_ITEM,
+                        pyatspi.ROLE_CHECK_MENU_ITEM,
+                        pyatspi.ROLE_RADIO_MENU_ITEM]:
+                msg = 'EVENT MANAGER: Ignoring event type due to role'
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return True
+
         if event.type.startswith('object:state-changed:showing'):
             if role not in [pyatspi.ROLE_ALERT,
                             pyatspi.ROLE_ANIMATION,
@@ -293,7 +301,8 @@ class EventManager:
                 toolkitName = app.toolkitName
             except:
                 toolkitName = None
-            if toolkitName in self._synchronousToolkits:
+            if toolkitName in self._synchronousToolkits \
+               or isinstance(e, input_event.MouseButtonEvent):
                 asyncMode = False
             script = _scriptManager.getScript(app, e.source)
             script.eventCache[e.type] = (e, time.time())
@@ -422,7 +431,7 @@ class EventManager:
         msg = 'EVENT MANAGER: registering listeners for: %s' % script
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        for eventType in list(script.listeners.keys()):
+        for eventType in script.listeners.keys():
             self._registerListener(eventType)
 
     def deregisterScriptListeners(self, script):
@@ -436,19 +445,19 @@ class EventManager:
         msg = 'EVENT MANAGER: deregistering listeners for: %s' % script
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        for eventType in list(script.listeners.keys()):
+        for eventType in script.listeners.keys():
             self._deregisterListener(eventType)
 
     def registerModuleListeners(self, listeners):
         """Register the listeners on behalf of the caller."""
 
-        for eventType, function in list(listeners.items()):
+        for eventType, function in listeners.items():
             self.registry.registerEventListener(function, eventType)
 
     def deregisterModuleListeners(self, listeners):
         """Deegister the listeners on behalf of the caller."""
 
-        for eventType, function in list(listeners.items()):
+        for eventType, function in listeners.items():
             self.registry.deregisterEventListener(function, eventType)
 
     def registerKeystrokeListener(self, function, mask=None, kind=None):
@@ -457,10 +466,10 @@ class EventManager:
         msg = 'EVENT MANAGER: registering keystroke listener function: %s' % function
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        if mask == None:
+        if mask is None:
             mask = list(range(256))
 
-        if kind == None:
+        if kind is None:
             kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
 
         self.registry.registerKeystrokeListener(function, mask=mask, kind=kind)
@@ -471,10 +480,10 @@ class EventManager:
         msg = 'EVENT MANAGER: deregistering keystroke listener function: %s' % function
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        if mask == None:
+        if mask is None:
             mask = list(range(256))
 
-        if kind == None:
+        if kind is None:
             kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
 
         self.registry.deregisterKeystrokeListener(
