@@ -38,6 +38,7 @@ TUTORIAL = '(tutorial and (pause + tutorial) or [])'
 MNEMONIC = '(mnemonic and (pause + mnemonic + lineBreak) or [])'
 
 BRAILLE_TEXT = '[Text(obj, asString(labelOrName or placeholderText), asString(eol), startOffset, endOffset)]\
+                + (invalid and [Region(" " + asString(invalid))])\
                 + (required and [Region(" " + asString(required))])\
                 + (readOnly and [Region(" " + asString(readOnly))])'
 
@@ -52,11 +53,13 @@ formatting = {
 
     'strings' : {
         'speech': {
+            'invalid': object_properties.INVALID_INDICATORS_SPEECH,
             'required': object_properties.STATE_REQUIRED_SPEECH,
             'readonly': object_properties.STATE_READ_ONLY_SPEECH,
             'insensitive': object_properties.STATE_INSENSITIVE_SPEECH,
             'checkbox': object_properties.CHECK_BOX_INDICATORS_SPEECH,
             'radiobutton': object_properties.RADIO_BUTTON_INDICATORS_SPEECH,
+            'switch': object_properties.SWITCH_INDICATORS_SPEECH,
             'togglebutton': object_properties.TOGGLE_BUTTON_INDICATORS_SPEECH,
             'expansion': object_properties.EXPANSION_INDICATORS_SPEECH,
             'nodelevel': object_properties.NODE_LEVEL_SPEECH,
@@ -72,19 +75,23 @@ formatting = {
             'required': object_properties.STATE_REQUIRED_BRAILLE,
             'readonly': object_properties.STATE_READ_ONLY_BRAILLE,
             'insensitive': object_properties.STATE_INSENSITIVE_BRAILLE,
+            'invalid': object_properties.INVALID_INDICATORS_BRAILLE,
             'checkbox': object_properties.CHECK_BOX_INDICATORS_BRAILLE,
             'radiobutton': object_properties.RADIO_BUTTON_INDICATORS_BRAILLE,
+            'switch': object_properties.SWITCH_INDICATORS_BRAILLE,
             'togglebutton': object_properties.TOGGLE_BUTTON_INDICATORS_BRAILLE,
             'expansion': object_properties.EXPANSION_INDICATORS_BRAILLE,
             'nodelevel': object_properties.NODE_LEVEL_BRAILLE,
             'nestinglevel': object_properties.NESTING_LEVEL_BRAILLE,
         },
         'sound': {
+            'invalid': object_properties.INVALID_INDICATORS_SOUND,
             'required': object_properties.STATE_REQUIRED_SOUND,
             'readonly': object_properties.STATE_READ_ONLY_SOUND,
             'insensitive': object_properties.STATE_INSENSITIVE_SOUND,
             'checkbox': object_properties.CHECK_BOX_INDICATORS_SOUND,
             'radiobutton': object_properties.RADIO_BUTTON_INDICATORS_SOUND,
+            'switch': object_properties.SWITCH_INDICATORS_SOUND,
             'togglebutton': object_properties.TOGGLE_BUTTON_INDICATORS_SOUND,
             'expansion': object_properties.EXPANSION_INDICATORS_SOUND,
             'multiselect': object_properties.STATE_MULTISELECT_SOUND,
@@ -125,6 +132,10 @@ formatting = {
         pyatspi.ROLE_ANIMATION: {
             'unfocused': 'labelAndName'
             },
+        pyatspi.ROLE_BLOCK_QUOTE: {
+            'focused' : 'leaving or (roleName + pause + nestingLevel)',
+            'unfocused': 'roleName + pause + nestingLevel + pause + displayedText',
+            },
         pyatspi.ROLE_CANVAS: {
             'focused': 'labelAndName + imageDescription + roleName + pause + positionInList',
             'unfocused': 'labelAndName + imageDescription + roleName + pause + positionInList',
@@ -136,8 +147,8 @@ formatting = {
             },
         pyatspi.ROLE_CHECK_BOX: {
             'focused': 'checkedState',
-            'unfocused': 'labelOrName + roleName + checkedState + required + availability + ' + MNEMONIC + ' + accelerator',
-            'basicWhereAmI': 'namedContainingPanel + labelOrName + roleName + checkedState + ' + MNEMONIC + ' + accelerator + required'
+            'unfocused': 'labelOrName + roleName + checkedState + required + pause + invalid + availability + ' + MNEMONIC + ' + accelerator',
+            'basicWhereAmI': 'namedContainingPanel + labelOrName + roleName + checkedState + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
             },
         pyatspi.ROLE_CHECK_MENU_ITEM: {
             'focused': 'checkedState',
@@ -154,8 +165,8 @@ formatting = {
             },
         pyatspi.ROLE_COMBO_BOX: {
             'focused': 'expandableState',
-            'unfocused': 'labelAndName + roleName + pause + positionInList + ' + MNEMONIC + ' + accelerator',
-            'basicWhereAmI': 'label + roleName + pause + name + positionInList + ' + MNEMONIC + ' + accelerator'
+            'unfocused': 'labelAndName + roleName + pause + (currentLineText + anyTextSelection or positionInList) + ' + MNEMONIC + ' + accelerator',
+            'basicWhereAmI': 'label + roleName + pause + name + (currentLineText + anyTextSelection or positionInList) + ' + MNEMONIC + ' + accelerator'
             },
         pyatspi.ROLE_DIAL: {
             'focused': 'value',
@@ -163,7 +174,7 @@ formatting = {
             'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
         pyatspi.ROLE_DIALOG: {
-            'focused': 'labelOrName',
+            'focused': 'labelOrName + roleName',
             'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
             },
         pyatspi.ROLE_DOCUMENT_FRAME: {
@@ -177,16 +188,16 @@ formatting = {
             },
         pyatspi.ROLE_ENTRY: {
             'focused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection',
-            'unfocused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection + ' + MNEMONIC,
-            'basicWhereAmI': 'labelOrName + readOnly + textRole + (textContent or placeholderText) + anyTextSelection + ' + MNEMONIC,
-            'detailedWhereAmI': 'labelOrName + readOnly + textRole + (textContentWithAttributes or placeholderText) + anyTextSelection + ' + MNEMONIC,
+            'unfocused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection + required + pause + invalid + ' + MNEMONIC,
+            'basicWhereAmI': 'labelOrName + readOnly + textRole + (textContent or placeholderText) + anyTextSelection + required + pause + invalid + ' + MNEMONIC,
+            'detailedWhereAmI': 'labelOrName + readOnly + textRole + (textContentWithAttributes or placeholderText) + anyTextSelection + required + pause + invalid + ' + MNEMONIC,
             },
         pyatspi.ROLE_FOOTER: {
             'unfocused': '(displayedText or name) + roleName',
         },
         pyatspi.ROLE_FORM: {
-            'focused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection',
-            'unfocused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection + ' + MNEMONIC,
+            'focused': 'leaving or (labelAndName + roleName)',
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + unrelatedLabels'
             },
         pyatspi.ROLE_FRAME: {
             'focused': 'labelOrName + roleName',
@@ -214,9 +225,13 @@ formatting = {
             'unfocused': 'labelAndName + unrelatedLabels'
             },
         pyatspi.ROLE_LABEL: {
-            'focused': 'labelAndName + allTextSelection + roleName',
-            'unfocused': 'labelAndName + allTextSelection + roleName',
-            'basicWhereAmI': 'labelAndName + allTextSelection + roleName'
+            'focused': 'displayedText + allTextSelection + roleName',
+            'unfocused': 'displayedText + allTextSelection + roleName',
+            'basicWhereAmI': 'displayedText + allTextSelection + roleName'
+            },
+        pyatspi.ROLE_LANDMARK: {
+            'focused': 'leaving or (labelAndName + roleName)',
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + unrelatedLabels'
             },
         pyatspi.ROLE_LAYERED_PANE: {
             'focused': 'labelAndName + roleName + availability + noShowingChildren',
@@ -229,7 +244,7 @@ formatting = {
             'basicWhereAmI': 'linkInfo + pause + siteDescription + pause + fileSize + pause + ' + MNEMONIC
             },
         pyatspi.ROLE_LIST: {
-            'focused': 'labelOrName + multiselectableState + numberOfChildren',
+            'focused' : 'leaving or (numberOfChildren + pause + nestingLevel)',
             'unfocused': 'labelOrName + pause + focusedItem + pause + multiselectableState + numberOfChildren + pause'
             },
         pyatspi.ROLE_LIST_BOX: {
@@ -306,7 +321,7 @@ formatting = {
             'basicWhereAmI': 'parentRoleName + pause + labelOrName + roleName + pause + positionInList + ' + MNEMONIC + ' + accelerator'
             },
         pyatspi.ROLE_PANEL: {
-            'focused': 'labelAndName + roleName',
+            'focused': 'leaving or (labelAndName + roleName)',
             'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + unrelatedLabels'
             },
         pyatspi.ROLE_PARAGRAPH: {
@@ -361,13 +376,13 @@ formatting = {
             },
         pyatspi.ROLE_SLIDER: {
             'focused': 'value',
-            'unfocused': 'labelOrName + roleName + value + required + availability + ' + MNEMONIC,
-            'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
+            'unfocused': 'labelOrName + roleName + value + required + pause + invalid + availability + ' + MNEMONIC,
+            'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
             },
         pyatspi.ROLE_SPIN_BUTTON: {
             'focused': 'name',
-            'unfocused': 'labelAndName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + required',
-            'basicWhereAmI': 'label + roleName + name + allTextSelection + ' + MNEMONIC + ' + accelerator + required'
+            'unfocused': 'labelAndName + allTextSelection + roleName + required + pause + invalid + availability + ' + MNEMONIC,
+            'basicWhereAmI': 'label + roleName + name + allTextSelection + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
             },
         pyatspi.ROLE_SPLIT_PANE: {
             'focused': 'value',
@@ -380,8 +395,14 @@ formatting = {
         'ROLE_STATIC': {
             'unfocused': '(displayedText or name) + roleName',
         },
+        'ROLE_SWITCH': {
+            'focused': 'switchState',
+            'unfocused': 'labelOrName + roleName + switchState + availability + ' + MNEMONIC + ' + accelerator',
+            'basicWhereAmI': 'labelOrName + roleName + switchState'
+            },
+
         pyatspi.ROLE_TABLE: {
-            'focused': 'labelAndName + pause + table',
+            'focused': 'leaving or (labelAndName + pause + table)',
             'unfocused': 'labelAndName + pause + table',
             'basicWhereAmI': 'labelAndName + pause + table'
             },
@@ -404,7 +425,7 @@ formatting = {
                               + cellCheckedState\
                               + (realActiveDescendantDisplayedText or imageDescription + image)\
                               + (expandableState and (expandableState + numberOfChildren))\
-                              + required)'
+                              + required + pause + invalid)'
             },
         pyatspi.ROLE_TABLE_ROW: {
             'focused': 'expandableState',
@@ -430,6 +451,10 @@ formatting = {
             'unfocused': 'labelOrName + roleName + (expandableState or toggleState) + availability + ' + MNEMONIC + ' + accelerator',
             'basicWhereAmI': 'labelOrName + roleName + (expandableState or toggleState)'
             },
+        pyatspi.ROLE_TOOL_BAR: {
+            'focused': 'labelAndName + roleName',
+            'unfocused': 'labelAndName + roleName',
+            },
         pyatspi.ROLE_TOOL_TIP: {
             'unfocused': 'labelAndName',
             'basicWhereAmI': 'labelAndName'
@@ -480,9 +505,9 @@ formatting = {
             },
         'default': {
             'focused':   '[Component(obj,\
-                                     asString(label + displayedText + value + roleName + required))]',
+                                     asString(label + displayedText + value + roleName + required + invalid))]',
             'unfocused': '[Component(obj,\
-                                     asString(label + displayedText + value + roleName + required))]',
+                                     asString(label + displayedText + value + roleName + required + invalid))]',
             },
         pyatspi.ROLE_ALERT: {
             'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
@@ -664,13 +689,18 @@ formatting = {
         #'REAL_ROLE_SCROLL_PANE': 'default'
         pyatspi.ROLE_SLIDER: {
             'unfocused': '[Component(obj,\
-                                     asString(labelOrName + value + roleName + required))]'
+                                     asString(labelOrName + value + roleName + required + invalid))]'
             },
         pyatspi.ROLE_SPIN_BUTTON: {
             'unfocused': '[Text(obj, asString(label), asString(eol))]\
                           + (required and [Region(" " + asString(required))] or [])\
                           + (readOnly and [Region(" " + asString(readOnly))] or [])'
             },
+        'ROLE_SWITCH' : {
+            'unfocused': '[Component(obj,\
+                                     asString((labelOrName or description) + roleName),\
+                                     indicator=asString(switchState))]'
+            },
         #pyatspi.ROLE_SPLIT_PANE: 'default'
         #pyatspi.ROLE_TABLE: 'default'
         pyatspi.ROLE_TABLE_CELL: {
@@ -748,7 +778,7 @@ formatting = {
         },
         pyatspi.ROLE_CHECK_BOX: {
             'focused': 'checkedState',
-            'unfocused': 'roleName + checkedState + required + availability',
+            'unfocused': 'roleName + checkedState + required + invalid + availability',
         },
         pyatspi.ROLE_CHECK_MENU_ITEM: {
             'focused': 'checkedState',
@@ -760,10 +790,10 @@ formatting = {
         },
         pyatspi.ROLE_DIAL: {
             'focused': 'percentage',
-            'unfocused': 'roleName + percentage + required + availability',
+            'unfocused': 'roleName + percentage + required + invalid + availability',
         },
         pyatspi.ROLE_ENTRY: {
-            'unfocused': 'roleName + readOnly + required + availability',
+            'unfocused': 'roleName + readOnly + required + invalid + availability',
         },
         pyatspi.ROLE_HEADING: {
             'focused': 'expandableState',
@@ -815,16 +845,20 @@ formatting = {
         },
         pyatspi.ROLE_SLIDER: {
             'focused': 'percentage',
-            'unfocused': 'roleName + percentage + required + availability',
+            'unfocused': 'roleName + percentage + required + invalid + availability',
         },
         pyatspi.ROLE_SPIN_BUTTON: {
             'focused': 'percentage',
-            'unfocused': 'roleName + availability + percentage + required',
+            'unfocused': 'roleName + availability + percentage + required + invalid',
         },
         pyatspi.ROLE_SPLIT_PANE: {
             'focused': 'percentage',
             'unfocused': 'roleName + percentage + availability',
         },
+        'ROLE_SWITCH': {
+            'focused': 'switchState',
+            'unfocused': 'roleName + switchState + availability',
+        },
         pyatspi.ROLE_TABLE_CELL: {
             'focused': 'expandableState',
             'unfocused': 'roleName + expandableState',
@@ -833,7 +867,7 @@ formatting = {
             'focused': 'expandableState',
         },
         pyatspi.ROLE_TEXT: {
-            'unfocused': 'roleName + readOnly + required + availability',
+            'unfocused': 'roleName + readOnly + required + invalid + availability',
         },
         pyatspi.ROLE_TOGGLE_BUTTON: {
             'focused': 'expandableState or toggleState',
