@@ -96,10 +96,18 @@ class Utilities(script_utilities.Utilities):
            and (self.isSpreadSheetCell(obj) or self.isTextDocumentCell(obj)):
             return ""
 
-        return text
+        # More bogusness from (at least) Calc combined with the aforementioned
+        # fallback-to-name behavior....
+        if self.isDocument(obj) and text == obj.name and obj.name.startswith("file:///"):
+            return ""
 
-    def isTextArea(self, obj):
-        return obj and obj.getRole() == pyatspi.ROLE_TEXT
+        # TODO - JD: Once the VCL script is completed and subclasses the
+        # appropriate toolkit scripts, this should not be needed.
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            labels = self.unrelatedLabels(obj, onlyShowing=False)
+            text = " ".join(map(self.displayedText, labels))
+
+        return text
 
     def isCellBeingEdited(self, obj):
         if not obj:
@@ -153,67 +161,6 @@ class Utilities(script_utilities.Utilities):
 
         return row, column, table
 
-    def getShowingCellsInRow(self, obj):
-        row, column, parentTable = self.getRowColumnAndTable(obj)
-        try:
-            table = parentTable.queryTable()
-        except:
-            return []
-
-        startIndex, endIndex = self.getTableRowRange(obj)
-        cells = []
-        for i in range(startIndex, endIndex):
-            cell = table.getAccessibleAt(row, i)
-            try:
-                showing = cell.getState().contains(pyatspi.STATE_SHOWING)
-            except:
-                continue
-            if showing:
-                cells.append(cell)
-
-        return cells
-
-    def getTableRowRange(self, obj):
-        """If this is spread sheet cell, return the start and end indices
-        of the spread sheet cells for the table that obj is in. Otherwise
-        return the complete range (0, parentTable.nColumns).
-
-        Arguments:
-        - obj: a table cell.
-
-        Returns the start and end table cell indices.
-        """
-
-        parent = obj.parent
-        try:
-            parentTable = parent.queryTable()
-        except:
-            return [-1, -1]
-
-        startIndex = 0
-        endIndex = parentTable.nColumns
-
-        if self.isSpreadSheetCell(obj):
-            extents = parent.queryComponent().getExtents(pyatspi.DESKTOP_COORDS)
-            y = extents.y
-            leftX = extents.x + 1
-            leftCell = \
-                parent.queryComponent().getAccessibleAtPoint(leftX, y, 0)
-            if leftCell:
-                table = leftCell.parent.queryTable()
-                index = self.cellIndex(leftCell)
-                startIndex = table.getColumnAtIndex(index)
-
-            rightX = extents.x + extents.width - 1
-            rightCell = \
-                parent.queryComponent().getAccessibleAtPoint(rightX, y, 0)
-            if rightCell:
-                table = rightCell.parent.queryTable()
-                index = self.cellIndex(rightCell)
-                endIndex = table.getColumnAtIndex(index) + 1
-
-        return [startIndex, endIndex]
-
     def rowHeadersForCell(self, obj):
         rowHeader, colHeader = self.getDynamicHeadersForCell(obj)
         if rowHeader:
@@ -260,27 +207,32 @@ class Utilities(script_utilities.Utilities):
         return rowHeader, colHeader
 
     def isSameObject(self, obj1, obj2, comparePaths=False, ignoreNames=False):
-        same = super().isSameObject(obj1, obj2, comparePaths, ignoreNames)
-        if not same or obj1 == obj2:
-            return same
+        if obj1 == obj2:
+            return True
 
-        # The document frame currently contains just the active page,
-        # resulting in false positives. So for paragraphs, rely upon
-        # the equality check.
-        if obj1.getRole() == obj2.getRole() == pyatspi.ROLE_PARAGRAPH:
+        try:
+            role1 = obj1.getRole()
+            role2 = obj2.getRole()
+        except:
             return False
 
-        # Handle the case of false positives in dialog boxes resulting
-        # from getIndexInParent() returning a bogus value. bgo#618790.
-        #
-        if not obj1.name \
-           and obj1.getRole() == pyatspi.ROLE_TABLE_CELL \
-           and obj1.getIndexInParent() == obj2.getIndexInParent() == -1:
-            top = self.topLevelObject(obj1)
-            if top and top.getRole() == pyatspi.ROLE_DIALOG:
-                same = False
+        if role1 != role2 or role1 == pyatspi.ROLE_PARAGRAPH:
+            return False
 
-        return same
+        try:
+            name1 = obj1.name
+            name2 = obj2.name
+        except:
+            return False
+
+        if name1 == name2:
+            if role1 == pyatspi.ROLE_FRAME:
+                return True
+            if role1 == pyatspi.ROLE_TABLE_CELL and not name1:
+                if self.isZombie(obj1) and self.isZombie(obj2):
+                    return False
+
+        return super().isSameObject(obj1, obj2, comparePaths, ignoreNames)
 
     def isLayoutOnly(self, obj):
         """Returns True if the given object is a container which has
@@ -289,19 +241,33 @@ class Utilities(script_utilities.Utilities):
         try:
             role = obj.getRole()
             childCount = obj.childCount
+            name = obj.name
         except:
-            role = None
-            childCount = 0
-
-        if role == pyatspi.ROLE_PANEL and childCount == 1:
-            if obj.name and obj.name == obj[0].name:
-                return True
-
-        if role == pyatspi.ROLE_LIST \
-           and obj.parent.getRole() == pyatspi.ROLE_COMBO_BOX:
+            msg = "SOFFICE: Exception getting properties of %s" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
             return True
 
-        return script_utilities.Utilities.isLayoutOnly(self, obj)
+        if role == pyatspi.ROLE_PANEL and childCount == 1 and name:
+            try:
+                child = obj[0]
+            except:
+                msg = "SOFFICE: Exception getting child of %s" % obj
+                debug.println(debug.LEVEL_INFO, msg, True)
+            else:
+                if child and child.name == name:
+                    return True
+
+        if role == pyatspi.ROLE_LIST:
+            try:
+                parentRole = obj.parent.getRole()
+            except:
+                msg = "SOFFICE: Exception getting parent role of %s" % obj
+                debug.println(debug.LEVEL_INFO, msg, True)
+            else:
+                if parentRole == pyatspi.ROLE_COMBO_BOX:
+                    return True
+
+        return super().isLayoutOnly(obj)
 
     def isAnInputLine(self, obj):
         if not obj:
@@ -407,35 +373,45 @@ class Utilities(script_utilities.Utilities):
 
         return results
 
-    def isFunctionalDialog(self, obj):
-        """Returns true if the window is functioning as a dialog."""
+    def validatedTopLevelObject(self, obj):
+        # TODO - JD: We cannot just override topLevelObject() because that will
+        # break flat review access to document content in LO using Gtk+ 3. That
+        # bug seems to be fixed in LO v5.3.0. When that version is released, this
+        # and hopefully other hacks can be removed.
+        window = super().topLevelObject(obj)
+        if not window or window.getIndexInParent() >= 0:
+            return window
 
-        # The OOo Navigator window looks like a dialog, acts like a
-        # dialog, and loses focus requiring the user to know that it's
-        # there and needs Alt+F6ing into.  But officially it's a normal
-        # window.
+        msg = "SOFFICE: %s's window %s has -1 indexInParent" % (obj, window)
+        debug.println(debug.LEVEL_INFO, msg, True)
 
-        # There doesn't seem to be (an efficient) top-down equivalent
-        # of utilities.hasMatchingHierarchy(). But OOo documents have
-        # root panes; this thing does not.
-        #
-        rolesList = [pyatspi.ROLE_FRAME,
-                     pyatspi.ROLE_PANEL,
-                     pyatspi.ROLE_PANEL,
-                     pyatspi.ROLE_TOOL_BAR,
-                     pyatspi.ROLE_PUSH_BUTTON]
+        for child in self._script.app:
+            if self.isSameObject(child, window):
+                window = child
+                break
 
-        if obj.getRole() != rolesList[0]:
-            # We might be looking at the child.
-            #
-            rolesList.pop(0)
+        try:
+            index = window.getIndexInParent()
+        except:
+            index = -1
 
-        while obj and obj.childCount and len(rolesList):
-            if obj.getRole() != rolesList.pop(0):
-                return False
-            obj = obj[0]
+        msg = "SOFFICE: Returning %s (index: %i)" % (window, index)
+        debug.println(debug.LEVEL_INFO, msg, True)
+        return window
 
-        return True
+    def commonAncestor(self, a, b):
+        ancestor = super().commonAncestor(a, b)
+        if ancestor or not (a and b):
+            return ancestor
+
+        windowA = self.validatedTopLevelObject(a)
+        windowB = self.validatedTopLevelObject(b)
+        if not self.isSameObject(windowA, windowB):
+            return None
+
+        msg = "SOFFICE: Adjusted ancestor %s and %s to %s" % (a, b, windowA)
+        debug.println(debug.LEVEL_INFO, msg, True)
+        return windowA
 
     def validParent(self, obj):
         """Returns the first valid parent/ancestor of obj. We need to do
@@ -512,7 +488,7 @@ class Utilities(script_utilities.Utilities):
     def isDrawingView(self, obj):
         """Returns True if obj is the Impress Drawing View."""
 
-        if obj and obj.getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
+        if self.isDocument(obj):
             return (":" in obj.name and "/" in obj.name)
 
         return False
@@ -560,8 +536,7 @@ class Utilities(script_utilities.Utilities):
             return None, None
 
         slidePane = taskPane = None
-        hasRole = lambda x: x and x.getRole() == pyatspi.ROLE_DOCUMENT_FRAME
-        if pyatspi.findAllDescendants(panes[0], hasRole):
+        if pyatspi.findAllDescendants(panes[0], self.isDocument):
             slidePane = panes[0]
             if len(panes) == 2:
                 taskPane = panes[1]
@@ -579,10 +554,10 @@ class Utilities(script_utilities.Utilities):
         Returns a (title, position, count) tuple.
         """
 
-        if obj.getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
+        if self.isDocument(obj):
             dv = obj
         else:
-            dv = self.ancestorWithRole(obj, [pyatspi.ROLE_DOCUMENT_FRAME], [])
+            dv = self.getContainingDocument(obj)
 
         if not dv or not self.isDrawingView(dv):
             return "", 0, 0
@@ -646,6 +621,62 @@ class Utilities(script_utilities.Utilities):
 
         return False
 
+    def containingComboBox(self, obj):
+        isComboBox = lambda x: x and x.getRole() == pyatspi.ROLE_COMBO_BOX
+        if isComboBox(obj):
+            comboBox = obj
+        else:
+            comboBox = pyatspi.findAncestor(obj, isComboBox)
+
+        if not comboBox:
+            return None
+
+        if not self.isZombie(comboBox):
+            return comboBox
+
+        try:
+            parent = comboBox.parent
+        except:
+            pass
+        else:
+            replicant = self.findReplicant(parent, comboBox)
+            if replicant and not self.isZombie(replicant):
+                comboBox = replicant
+
+        return comboBox
+
+    def isComboBoxSelectionChange(self, event):
+        comboBox = self.containingComboBox(event.source)
+        if not comboBox:
+            return False
+
+        lastKey, mods = self.lastKeyAndModifiers()
+        if not lastKey in ["Down", "Up"]:
+            return False
+
+        return True
+
+    def isComboBoxNoise(self, event):
+        role = event.source.getRole()
+        if role == pyatspi.ROLE_TEXT and event.type.startswith("object:text-"):
+            return self.isComboBoxSelectionChange(event)
+
+        return False
+
+    def isPresentableTextChangedEventForLocusOfFocus(self, event):
+        if self.isComboBoxNoise(event):
+            msg = "SOFFICE: Event is believed to be combo box noise"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
+
+        return super().isPresentableTextChangedEventForLocusOfFocus(event)
+
+    def isReadOnlyTextArea(self, obj):
+        if not super().isReadOnlyTextArea(obj):
+            return False
+
+        return not self.inDocumentContent(obj)
+
     def isSelectedTextDeletionEvent(self, event):
         if event.type.startswith("object:state-changed:selected") and not event.detail1:
             return self.isDead(orca_state.locusOfFocus) and self.lastInputEventWasDelete()
@@ -666,11 +697,17 @@ class Utilities(script_utilities.Utilities):
         if not obj:
             return []
 
+        role = obj.getRole()
+        isSelection = lambda x: x and 'Selection' in pyatspi.listInterfaces(x)
+        if not isSelection(obj) and role == pyatspi.ROLE_COMBO_BOX:
+            child = pyatspi.findDescendant(obj, isSelection)
+            if child:
+                return super().selectedChildren(child)
+
         # Things only seem broken for certain tables, e.g. the Paths table.
         # TODO - JD: File the LibreOffice bugs and reference them here.
-        if obj.getRole() != pyatspi.ROLE_TABLE \
-           or self.isSpreadSheetCell(obj):
-            return script_utilities.Utilities.selectedChildren(self, obj)
+        if role != pyatspi.ROLE_TABLE or self.isSpreadSheetCell(obj):
+            return super().selectedChildren(obj)
 
         try:
             selection = obj.querySelection()
@@ -683,3 +720,25 @@ class Utilities(script_utilities.Utilities):
                 children.append(obj[i])
 
         return children
+
+    def getFirstCaretPosition(self, obj):
+        try:
+            text = obj.queryText()
+        except:
+            if obj and obj.childCount:
+                return self.getFirstCaretPosition(obj[0])
+
+        return obj, 0
+
+    def shouldReadFullRow(self, obj):
+        if self._script._lastCommandWasStructNav:
+            return False
+
+        lastKey, mods = self.lastKeyAndModifiers()
+        if lastKey in ["Tab", "ISO_Left_Tab"]:
+            return False
+
+        if not super().shouldReadFullRow(obj):
+            return False
+
+        return self.cellRowChanged(obj)
