@@ -27,6 +27,7 @@ __license__   = "LGPL"
 
 import pyatspi
 
+import orca.debug as debug
 import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.scripts.default as default
@@ -54,25 +55,20 @@ class Script(default.Script):
         if self.utilities.isToggleDescendantOfComboBox(newFocus):
             isComboBox = lambda x: x and x.getRole() == pyatspi.ROLE_COMBO_BOX
             newFocus = pyatspi.findAncestor(newFocus, isComboBox) or newFocus
+            orca.setLocusOfFocus(event, newFocus, False)
 
         super().locusOfFocusChanged(event, oldFocus, newFocus)
 
     def onActiveDescendantChanged(self, event):
         """Callback for object:active-descendant-changed accessibility events."""
 
-        role = event.source.getRole()
+        if not self.utilities.isTypeahead(orca_state.locusOfFocus):
+            super().onActiveDescendantChanged(event)
+            return
 
-        try:
-            focusedRole = orca_state.locusOfFocus.getRole()
-        except:
-            pass
-        else:
-            # This is very likely typeahead search and not a real focus change.
-            tableRoles = [pyatspi.ROLE_TABLE, pyatspi.ROLE_TREE_TABLE]
-            if focusedRole == pyatspi.ROLE_TEXT and role in tableRoles:
-                orca.setLocusOfFocus(event, event.source, False)
-
-        default.Script.onActiveDescendantChanged(self, event)
+        msg = "GTK: locusOfFocus believed to be typeahead. Presenting change."
+        debug.println(debug.LEVEL_INFO, msg, True)
+        self.presentObject(event.any_data)
 
     def onCheckedChanged(self, event):
         """Callback for object:state-changed:checked accessibility events."""
@@ -128,11 +124,6 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
-        # https://bugzilla.gnome.org/show_bug.cgi?id=748311
-        if role == pyatspi.ROLE_TERMINAL:
-            orca.setLocusOfFocus(event, event.source)
-            return
-
         # https://bugzilla.gnome.org/show_bug.cgi?id=720987
         if role == pyatspi.ROLE_TABLE_COLUMN_HEADER:
             orca.setLocusOfFocus(event, event.source)
@@ -140,7 +131,11 @@ class Script(default.Script):
 
         # https://bugzilla.gnome.org/show_bug.cgi?id=720989
         if role == pyatspi.ROLE_MENU == event.source.parent.getRole():
-            orca.setLocusOfFocus(event, event.source)
+            if event.source.name:
+                orca.setLocusOfFocus(event, event.source)
+            else:
+                msg = "GTK: Nameless menu with parent %s" % event.source.parent
+                debug.println(debug.LEVEL_INFO, msg, True)
             return
 
         # Unfiled, but a similar case of the above issue with combo boxes.
@@ -217,9 +212,19 @@ class Script(default.Script):
             super().onSelectionChanged(event)
             return
 
+        isFocused = event.source.getState().contains(pyatspi.STATE_FOCUSED)
         role = event.source.getRole()
-        if role == pyatspi.ROLE_COMBO_BOX \
-           and not event.source.getState().contains(pyatspi.STATE_FOCUSED):
+        if role == pyatspi.ROLE_COMBO_BOX and not isFocused:
+            return
+
+        if not isFocused and self.utilities.isTypeahead(orca_state.locusOfFocus):
+            msg = "GTK: locusOfFocus believed to be typeahead. Presenting change."
+            debug.println(debug.LEVEL_INFO, msg, True)
+
+            selectedChildren = self.utilities.selectedChildren(event.source)
+            for child in selectedChildren:
+                if not self.utilities.isLayoutOnly(child):
+                    self.presentObject(child)
             return
 
         if role == pyatspi.ROLE_LAYERED_PANE \
