@@ -541,6 +541,24 @@ class Script(default.Script):
             obj, characterOffset = self.utilities.getCaretContext()
         else:
             characterOffset = offset
+        priorObj, priorOffset = self.utilities.getPriorContext()
+
+        # TODO - JD: This is sad, but it's better than the old, broken
+        # clumpUtterances(). We really need to fix the speechservers'
+        # SayAll support. In the meantime, the generators should be
+        # providing one ACSS per string.
+        def _parseUtterances(utterances):
+            elements, voices = [], []
+            for u in utterances:
+                if isinstance(u, list):
+                    e, v = _parseUtterances(u)
+                    elements.extend(e)
+                    voices.extend(v)
+                elif isinstance(u, str):
+                    elements.append(u)
+                elif isinstance(u, ACSS):
+                    voices.append(u)
+            return elements, voices
 
         self._inSayAll = True
         done = False
@@ -551,15 +569,15 @@ class Script(default.Script):
                 contents = self.utilities.getLineContentsAtOffset(obj, characterOffset)
             self._sayAllContents = contents
             for content in contents:
-                obj, startOffset, endOffset, text = content
-                utterances = self.speechGenerator.generateContents([content], eliminatePauses=True)
+                if self.utilities.isInferredLabelForContents(content, contents):
+                    continue
 
-                # TODO - JD: This is sad, but it's better than the old, broken
-                # clumpUtterances(). We really need to fix the speechservers'
-                # SayAll support. In the meantime, the generators should be
-                # providing one ACSS per string.
-                elements = list(filter(lambda x: isinstance(x, str), utterances[0]))
-                voices = list(filter(lambda x: isinstance(x, ACSS), utterances[0]))
+                obj, startOffset, endOffset, text = content
+                utterances = self.speechGenerator.generateContents(
+                    [content], eliminatePauses=True, priorObj=priorObj)
+                priorObj = obj
+
+                elements, voices = _parseUtterances(utterances)
                 if len(elements) != len(voices):
                     continue
 
@@ -719,16 +737,17 @@ class Script(default.Script):
 
         return self.utilities.isFocusModeWidget(obj)
 
-    def speakContents(self, contents):
+    def speakContents(self, contents, **args):
         """Speaks the specified contents."""
 
-        utterances = self.speechGenerator.generateContents(contents)
+        utterances = self.speechGenerator.generateContents(contents, **args)
         speech.speak(utterances)
 
     def sayCharacter(self, obj):
         """Speaks the character at the current caret position."""
 
-        if not self._lastCommandWasCaretNav:
+        if not self._lastCommandWasCaretNav \
+           and not self.utilities.isContentEditableWithEmbeddedObjects(obj):
             super().sayCharacter(obj)
             return
 
@@ -753,7 +772,8 @@ class Script(default.Script):
     def sayWord(self, obj):
         """Speaks the word at the current caret position."""
 
-        if not self._lastCommandWasCaretNav:
+        if not self._lastCommandWasCaretNav \
+           and not self.utilities.isContentEditableWithEmbeddedObjects(obj):
             super().sayWord(obj)
             return
 
@@ -766,19 +786,29 @@ class Script(default.Script):
     def sayLine(self, obj):
         """Speaks the line at the current caret position."""
 
-        if not (self._lastCommandWasCaretNav or self._lastCommandWasStructNav):
+        if not (self._lastCommandWasCaretNav or self._lastCommandWasStructNav) \
+           and not self.utilities.isContentEditableWithEmbeddedObjects(obj):
             super().sayLine(obj)
             return
 
+        priorObj = None
+        if self._lastCommandWasCaretNav:
+            priorObj, priorOffset = self.utilities.getPriorContext()
+
         obj, offset = self.utilities.getCaretContext(documentFrame=None)
-        self.speakContents(self.utilities.getLineContentsAtOffset(obj, offset))
+        contents = self.utilities.getLineContentsAtOffset(obj, offset)
+        self.speakContents(contents, priorObj=priorObj)
 
     def presentObject(self, obj, **args):
+        priorObj = None
+        if self._lastCommandWasCaretNav:
+            priorObj, priorOffset = self.utilities.getPriorContext()
+
         offset = args.get("offset", 0)
         contents = self.utilities.getObjectContentsAtOffset(obj, offset)
         self.displayContents(contents)
-        self.speakContents(contents)
-
+        self.speakContents(contents, priorObj=priorObj)
+ 
     def updateBraille(self, obj, **args):
         """Updates the braille display to show the given object."""
 
@@ -787,14 +817,15 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, "BRAILLE: disabled", True)
             return
 
-        if self._inFocusMode or not self.utilities.inDocumentContent():
-            msg = "WEB: updating braille for non-browse-mode object %s" % obj
+        if not self.utilities.inDocumentContent(obj):
+            msg = "WEB: updating braille for non-document object %s" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
             super().updateBraille(obj, **args)
             return
 
         if not self._lastCommandWasCaretNav \
            and not self._lastCommandWasStructNav \
+           and not self.utilities.isContentEditableWithEmbeddedObjects(obj) \
            and not self.utilities.lastInputEventWasCaretNavWithSelection():
             msg = "WEB: updating braille for unhandled navigation type %s" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
@@ -1201,6 +1232,10 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, obj)
             return True
 
+        msg = "WEB: Setting locus of focus to context obj %s (no notification)" % obj
+        debug.println(debug.LEVEL_INFO, msg, True)
+        orca.setLocusOfFocus(event, obj, False)
+
         self.updateBraille(obj)
         if state.contains(pyatspi.STATE_FOCUSABLE) and not self.utilities.isDocument(obj):
             msg = "WEB: Not doing SayAll due to focusable context obj %s" % obj
@@ -1249,8 +1284,8 @@ class Script(default.Script):
         if self._lastCommandWasMouseButton:
             msg = "WEB: Event handled: Last command was mouse button"
             debug.println(debug.LEVEL_INFO, msg, True)
-            orca.setLocusOfFocus(event, event.source)
             self.utilities.setCaretContext(event.source, event.detail1)
+            orca.setLocusOfFocus(event, event.source)
             return True
 
         if self.utilities.inFindToolbar():
@@ -1285,24 +1320,24 @@ class Script(default.Script):
         if self.utilities.caretMovedToSamePageFragment(event):
             msg = "WEB: Event handled: Caret moved to fragment"
             debug.println(debug.LEVEL_INFO, msg, True)
-            orca.setLocusOfFocus(event, obj)
             self.utilities.setCaretContext(obj, offset)
+            orca.setLocusOfFocus(event, obj)
             return True
 
         if self.utilities.lastInputEventWasPageNav() \
            and not self.utilities.isLink(event.source):
             msg = "WEB: Event handled: Caret moved due to scrolling"
             debug.println(debug.LEVEL_INFO, msg, True)
-            orca.setLocusOfFocus(event, obj)
             self.utilities.setCaretContext(obj, offset)
+            orca.setLocusOfFocus(event, obj)
             return True
 
         if self.utilities.isContentEditableWithEmbeddedObjects(event.source):
             msg = "WEB: In content editable with embedded objects"
             debug.println(debug.LEVEL_INFO, msg, True)
             notify = orca_state.locusOfFocus == orca_state.activeWindow
-            orca.setLocusOfFocus(event, event.source, notify)
             self.utilities.setCaretContext(obj, offset)
+            orca.setLocusOfFocus(event, event.source, notify)
             return False
 
         text = self.utilities.queryNonEmptyText(event.source)
@@ -1310,8 +1345,8 @@ class Script(default.Script):
             if event.source.getRole() == pyatspi.ROLE_LINK:
                 msg = "WEB: Event handled: Was for non-text link"
                 debug.println(debug.LEVEL_INFO, msg, True)
-                orca.setLocusOfFocus(event, event.source)
                 self.utilities.setCaretContext(event.source, event.detail1)
+                orca.setLocusOfFocus(event, event.source)
             else:
                 msg = "WEB: Event ignored: Was for non-text non-link"
                 debug.println(debug.LEVEL_INFO, msg, True)
@@ -1336,8 +1371,8 @@ class Script(default.Script):
 
             msg = "WEB: Setting locusOfFocus, context to: %s, %i" % (obj, offset)
             debug.println(debug.LEVEL_INFO, msg, True)
-            orca.setLocusOfFocus(event, obj)
             self.utilities.setCaretContext(obj, offset)
+            orca.setLocusOfFocus(event, obj)
             return True
 
         if self.utilities.treatEventAsSpinnerValueChange(event):
@@ -1349,8 +1384,8 @@ class Script(default.Script):
 
         if not _settingsManager.getSetting('caretNavigationEnabled') \
            or self._inFocusMode or isEditable:
-            orca.setLocusOfFocus(event, event.source, False)
             self.utilities.setCaretContext(event.source, event.detail1)
+            orca.setLocusOfFocus(event, event.source, False)
             msg = "WEB: Setting locusOfFocus, context to: %s, %i" % \
                   (event.source, event.detail1)
             debug.println(debug.LEVEL_INFO, msg, True)
