@@ -49,30 +49,35 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return super()._generateAncestors(obj, **args)
 
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_LINK:
-            return []
+        result = []
+        priorObj = args.get('priorObj')
+        if priorObj and self._script.utilities.inDocumentContent(priorObj):
+            priorDoc = self._script.utilities.getDocumentForObject(priorObj)
+            doc = self._script.utilities.getDocumentForObject(obj)
+            if priorDoc != doc and not self._script.utilities.getDocumentForObject(doc):
+                result = [super()._generateName(doc)]
 
-        if self._script.utilities.isLandmark(obj):
-            return []
-
-        if self._script.utilities.isMath(obj):
-            return []
+        if self._script.utilities.isLink(obj) \
+           or self._script.utilities.isLandmark(obj) \
+           or self._script.utilities.isMath(obj):
+            return result
 
         args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_FRAME,
                                pyatspi.ROLE_DOCUMENT_WEB,
                                pyatspi.ROLE_EMBEDDED,
                                pyatspi.ROLE_INTERNAL_FRAME,
-                               pyatspi.ROLE_FORM,
                                pyatspi.ROLE_MATH,
                                pyatspi.ROLE_MENU_BAR,
                                pyatspi.ROLE_TOOL_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
                              pyatspi.ROLE_LABEL,
+                             pyatspi.ROLE_LINK,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TEXT]
 
-        return super()._generateAncestors(obj, **args)
+        result.extend(super()._generateAncestors(obj, **args))
+
+        return result
 
     def _generateAllTextSelection(self, obj, **args):
         if self._script.utilities.isZombie(obj) \
@@ -153,9 +158,21 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return super()._generateLabelOrName(obj, **args)
 
     def _generateName(self, obj, **args):
-        if self._script.utilities.isTextBlockElement(obj):
+        if self._script.utilities.isTextBlockElement(obj) \
+           and not self._script.utilities.isLandmark(obj):
             return []
 
+        # TODO - JD: Once the formatting strings are vastly cleaned up
+        # or simply removed, hacks like this won't be needed.
+        role = args.get('role', obj.getRole())
+        if role in [pyatspi.ROLE_COMBO_BOX, pyatspi.ROLE_SPIN_BUTTON]:
+            return super()._generateName(obj, **args)
+
+        if self._script.utilities.inDocumentContent(obj) and obj.name:
+            result = [obj.name]
+            result.extend(self.voice(speech_generator.DEFAULT))
+            return result
+
         return super()._generateName(obj, **args)
 
     def _generateLabel(self, obj, **args):
@@ -210,15 +227,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
     def _generateTextRole(self, obj, **args):
         return self._generateRoleName(obj, **args)
 
-    def getLocalizedRoleName(self, obj, role=None):
+    def getLocalizedRoleName(self, obj, **args):
         if not self._script.utilities.inDocumentContent(obj):
-            return super().getLocalizedRoleName(obj, role)
+            return super().getLocalizedRoleName(obj, **args)
 
         roledescription = self._script.utilities.getRoleDescription(obj)
         if roledescription:
             return roledescription
 
-        return super().getLocalizedRoleName(obj, role)
+        return super().getLocalizedRoleName(obj, **args)
 
     def _generateRoleName(self, obj, **args):
         if not self._script.utilities.inDocumentContent(obj):
@@ -234,14 +251,16 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return result
 
         role = args.get('role', obj.getRole())
-        force = args.get('force', False)
+        enabled, disabled = self._getEnabledAndDisabledContextRoles()
+        if role in disabled:
+            return []
 
+        force = args.get('force', False)
         start = args.get('startOffset')
         end = args.get('endOffset')
 
         if not force:
             doNotSpeak = [pyatspi.ROLE_FOOTER,
-                          pyatspi.ROLE_FORM,
                           pyatspi.ROLE_LABEL,
                           pyatspi.ROLE_MENU_ITEM,
                           pyatspi.ROLE_PARAGRAPH,
@@ -272,18 +291,18 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 if text and end not in [None, text.characterCount]:
                     return []
             if role not in doNotSpeak:
-                result.append(self.getLocalizedRoleName(obj, role))
+                result.append(self.getLocalizedRoleName(obj, **args))
                 result.extend(acss)
 
         elif role == pyatspi.ROLE_HEADING:
             level = self._script.utilities.headingLevel(obj)
             if level:
                 result.append(object_properties.ROLE_HEADING_LEVEL_SPEECH % {
-                    'role': self.getLocalizedRoleName(obj, role),
+                    'role': self.getLocalizedRoleName(obj, **args),
                     'level': level})
                 result.extend(acss)
             else:
-                result.append(self.getLocalizedRoleName(obj, role))
+                result.append(self.getLocalizedRoleName(obj, **args))
                 result.extend(acss)
 
         elif self._script.utilities.isLink(obj):
@@ -292,13 +311,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.extend(acss)
             else:
                 if self._script.utilities.hasUselessCanvasDescendant(obj):
-                    result.append(self.getLocalizedRoleName(obj, pyatspi.ROLE_IMAGE))
+                    result.append(self.getLocalizedRoleName(obj, role=pyatspi.ROLE_IMAGE))
                     result.extend(acss)
-                result.append(self.getLocalizedRoleName(obj, role))
+                result.append(self.getLocalizedRoleName(obj, **args))
                 result.extend(acss)
 
-        elif role not in doNotSpeak:
-            result.append(self.getLocalizedRoleName(obj, role))
+        elif role not in doNotSpeak and args.get('priorObj') != obj:
+            result.append(self.getLocalizedRoleName(obj, **args))
             result.extend(acss)
 
         index = args.get('index', 0)
@@ -397,6 +416,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
+        if self._script.utilities.isEditableComboBox(obj):
+            return []
+
         position = self._script.utilities.getPositionInSet(obj)
         total = self._script.utilities.getSetSize(obj)
         if position is None or total is None:
@@ -416,6 +438,20 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         result.extend(self.voice(speech_generator.SYSTEM))
         return result
 
+    def _generateTableCellRow(self, obj, **args):
+        if not self._script.inFocusMode():
+            return super()._generateTableCellRow(obj, **args)
+
+        if not self._script.utilities.shouldReadFullRow(obj):
+            return self._generateRealTableCell(obj, **args)
+
+        isRow = lambda x: x and x.getRole() == pyatspi.ROLE_TABLE_ROW
+        row = pyatspi.findAncestor(obj, isRow)
+        if row and row.name:
+            return self.generate(row)
+
+        return super()._generateTableCellRow(obj, **args)
+
     def generateSpeech(self, obj, **args):
         if not self._script.utilities.inDocumentContent(obj):
             msg = "WEB: %s is not in document content. Calling default speech generator." % obj
@@ -432,15 +468,21 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
         elif self._script.utilities.isAnchor(obj):
             oldRole = 'ROLE_STATIC'
-        elif self._script.utilities.treatAsDiv(obj):
+        elif self._script.utilities.treatAsDiv(obj, offset=args.get('startOffset')):
             oldRole = self._overrideRole(pyatspi.ROLE_SECTION, args)
         else:
             oldRole = self._overrideRole(self._getAlternativeRole(obj, **args), args)
 
-        result.extend(super().generateSpeech(obj, **args))
-        result = list(filter(lambda x: x, result))
-        self._restoreRole(oldRole, args)
+        if not 'priorObj' in args:
+            args['priorObj'] = self._script.utilities.getPriorContext()[0]
 
+        if self._script.utilities.isLabellingContents(obj):
+            result = list(filter(lambda x: x, self.generateContext(obj, **args)))
+
+        if not result:
+            result = list(filter(lambda x: x, super().generateSpeech(obj, **args)))
+
+        self._restoreRole(oldRole, args)
         msg = "WEB: Speech generation for document object %s complete:" % obj
         debug.println(debug.LEVEL_INFO, msg)
         for element in result:
@@ -466,6 +508,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 index=i, total=len(contents), **args)
             if utterance and utterance[0]:
                 result.append(utterance)
+                args['priorObj'] = obj
 
         if not result:
             if self._script.inSayAll():
